@@ -26,6 +26,7 @@
 #endif
 
 #include <string>
+#include <gio/gio.h>
 #include <ibus.h>
 #include <zhuyin.h>
 #include "ZYUtil.h"
@@ -33,11 +34,9 @@
 
 namespace ZY {
 
-class Bus;
-
-class Config : public Object {
+class Config {
 protected:
-    Config (Bus & bus, const std::string & name);
+    Config (const std::string & name);
     virtual ~Config (void);
 
 public:
@@ -47,6 +46,7 @@ public:
     gboolean isZhuyin(void) const               { return m_is_zhuyin; }
     ZhuyinScheme zhuyinScheme (void) const      { return m_zhuyin_scheme; }
     FullPinyinScheme pinyinScheme (void) const  { return m_pinyin_scheme; }
+    gboolean needTone (void) const              { return m_need_tone; }
     gboolean initChinese (void) const           { return m_init_chinese; }
     gboolean initFullWidth (void) const         { return m_init_full_width; }
     gboolean initTradChinese (void) const       { return m_init_trad_chinese; }
@@ -65,18 +65,18 @@ protected:
     void initDefaultValues (void);
 
     virtual void readDefaultValues (void);
-    virtual gboolean valueChanged (const std::string  &section,
+    virtual gboolean valueChanged (const std::string  &schema_id,
                                    const std::string  &name,
                                    GVariant           *value);
 private:
-    static void valueChangedCallback (IBusConfig     *config,
-                                      const gchar    *section,
+    static void valueChangedCallback (GSettings      *settings,
                                       const gchar    *name,
-                                      GVariant       *value,
                                       Config         *self);
 
 protected:
-    std::string m_section;
+    GSettings *m_settings;
+    std::string m_schema_id;
+
     zhuyin_option_t m_option;
     zhuyin_option_t m_option_mask;
 
@@ -86,6 +86,7 @@ protected:
     gboolean m_is_zhuyin;
     ZhuyinScheme m_zhuyin_scheme;
     FullPinyinScheme m_pinyin_scheme;
+    gboolean m_need_tone;
 
     gboolean m_init_chinese;
     gboolean m_init_full_width;
@@ -104,24 +105,33 @@ protected:
 static inline bool
 normalizeGVariant (GVariant *value, bool defval)
 {
-    if (value == NULL || g_variant_classify (value) != G_VARIANT_CLASS_BOOLEAN)
+    if (value == NULL ||
+        g_variant_classify (value) != G_VARIANT_CLASS_BOOLEAN) {
+        g_warn_if_reached ();
         return defval;
+    }
     return g_variant_get_boolean (value);
 }
 
 static inline gint
 normalizeGVariant (GVariant *value, gint defval)
 {
-    if (value == NULL || g_variant_classify (value) != G_VARIANT_CLASS_INT32)
+    if (value == NULL ||
+        g_variant_classify (value) != G_VARIANT_CLASS_INT32) {
+        g_warn_if_reached ();
         return defval;
+    }
     return g_variant_get_int32 (value);
 }
 
 static inline std::string
 normalizeGVariant (GVariant *value, const std::string &defval)
 {
-    if (value == NULL || g_variant_classify (value) != G_VARIANT_CLASS_STRING)
+    if (value == NULL ||
+        g_variant_classify (value) != G_VARIANT_CLASS_STRING) {
+        g_warn_if_reached ();
         return defval;
+    }
     return g_variant_get_string (value, NULL);
 }
 
